#!/bin/bash
#
# Script-Version: 0.1
# Author: Stefan Koehler ( http://www.soocs.de )
# Description: Shell script to setup and emulate I/O latency issue for Oracle LGWR with help of device mappers delay target

PARM=$1

if [[ ${PARM} = '-h' ]] || [[ ${PARM} = '--help' ]] || [[ $# -eq 0 ]] ; then
  echo -e "Usage: $0 {setup|run}"
  echo -e "setup - Initial setup dm devices before emulating I/O latency issue for Oracle LGWR" 
  echo -e "run - Emulate I/O latency issue for Oracle LGWR" 
  exit 1
fi;

#Format
CR=`echo $'\n.'`
CR=${CR%.}

# Parameter I/O latency for Oracle Redo
NOLATENCY=0
LATENCY=10
#LATENCY=30

if [[ ${PARM} = 'setup' ]] ; then

  # Oracle Orig Redo
  umount /oracle/origlog
  size_orig=$(blockdev --getsize /dev/mapper/vgoracle-lvoriglog)
  echo "0 $size_orig delay /dev/mapper/vgoracle-lvoriglog 0 $NOLATENCY" | dmsetup create lvoriglog_delay
  mount /dev/mapper/lvoriglog_delay /oracle/origlog

  # Oracle Mirr Redo
  umount /oracle/mirrlog
  size_mirr=$(blockdev --getsize /dev/mapper/vgoracle-lvmirrlog) 
  echo "0 $size_mirr delay /dev/mapper/vgoracle-lvmirrlog 0 $NOLATENCY" | dmsetup create lvmirrlog_delay
  mount /dev/mapper/lvmirrlog_delay /oracle/mirrlog

  exit 0

elif [[ ${PARM} = 'run' ]]; then

  size_orig=$(blockdev --getsize /dev/mapper/vgoracle-lvoriglog)
  size_mirr=$(blockdev --getsize /dev/mapper/vgoracle-lvmirrlog) 

  # Introduce I/O latency problem
  read -p "Press any key to continue to emulate LGWR I/O latency problem ... $CR" -n1 -s

  # Oracle Orig Redo
  dmsetup suspend lvoriglog_delay
  dmsetup reload lvoriglog_delay --table "0 $size_orig delay 252:5 0 $LATENCY"
  dmsetup resume lvoriglog_delay

  # Oracle Mirr Redo
  dmsetup suspend lvmirrlog_delay
  dmsetup reload lvmirrlog_delay --table "0 $size_mirr delay 252:6 0 $LATENCY"
  dmsetup resume lvmirrlog_delay

  # Fix I/O latency problem
  read -p "Press any key to continue to fix LGWR I/O latency problem ... $CR" -n1 -s

  # Oracle Orig Redo
  dmsetup suspend lvoriglog_delay
  dmsetup reload lvoriglog_delay --table "0 $size_orig delay 252:5 0 $NOLATENCY"
  dmsetup resume lvoriglog_delay

  # Oracle Mirr Redo
  dmsetup suspend lvmirrlog_delay
  dmsetup reload lvmirrlog_delay --table "0 $size_mirr delay 252:6 0 $NOLATENCY"
  dmsetup resume lvmirrlog_delay

else

  echo -e "Usage: $0 {setup|run}"
  echo -e "setup - Initial setup dm devices before emulating I/O latency issue for Oracle LGWR"
  echo -e "run - Emulate I/O latency issue for Oracle LGWR"
  exit 1

fi;

